package com.ningdatech.pmapi.scheduler.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.StopWatch;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.sys.enumeration.ProjectEarlyWarningStatusEnum;
import com.ningdatech.pmapi.sys.model.entity.ProjectEarlyWarning;
import com.ningdatech.pmapi.sys.service.IProjectEarlyWarningService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author ZPF
 * @date 2023/8/3 上午9:53
 * 预警填报 超时任务
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class EarlyWarningProjectTask {

    private final IProjectService projectService;

    private final IProjectEarlyWarningService projectEarlyWarningService;

    @Value("${hostname}")
    private String HOST_NAME;

    @Scheduled(fixedDelay = 6000 * 120)
    public void doEarlyWarningDeclared() throws UnknownHostException {
        if (!HOST_NAME.equals(InetAddress.getLocalHost().getHostName())) {
            return;
        }

        log.info("=========== 预警项目维度任务开始 ========");
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();

        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE));

        if(CollUtil.isEmpty(projects)){
            log.info("系统没有项目 退出任务");
            stopWatch.stop();
            log.info("=========== 预警项目维度任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
            return;
        }
        List<ProjectEarlyWarning> warnings = projectEarlyWarningService.list();
        List<String> warningProjectCodes = warnings.stream().map(ProjectEarlyWarning::getProjectCode)
                .collect(Collectors.toList());

        //1.先补全 异常项目管理中 不存在的项目
        List<ProjectEarlyWarning> toAdd = projects.stream()
                .filter(p -> {
                    if(CollUtil.isEmpty(warningProjectCodes)){
                        return Boolean.TRUE;
                    }
                    if(!warningProjectCodes.contains(p.getProjectCode())){
                        return Boolean.TRUE;
                    }
                    return Boolean.FALSE;
                })
                .map(p -> {
                    ProjectEarlyWarning earlyWarning = new ProjectEarlyWarning();
                    earlyWarning.setAreaCode(p.getAreaCode());
                    earlyWarning.setBuildOrgCode(p.getBuildOrgCode());
                    earlyWarning.setProjectCode(p.getProjectCode());
                    earlyWarning.setCreateOn(LocalDateTime.now());
                    earlyWarning.setStatus(ProjectEarlyWarningStatusEnum.NORMAL.name());
                    earlyWarning.setRuleType(null);
                    return earlyWarning;
                })
                .collect(Collectors.toList());

        if(CollUtil.isNotEmpty(toAdd)){
            projectEarlyWarningService.saveBatch(toAdd);
        }

        //2.去查询 各个报警 是否已经正常
        List<ProjectEarlyWarning> trueWarnings = warnings.stream().filter(w -> Objects.nonNull(w.getNormal()) && !w.getNormal()).collect(Collectors.toList());
        if(CollUtil.isEmpty(trueWarnings)){
            log.info("没有异常的项目 要去查询");
            stopWatch.stop();
            log.info("=========== 预警项目维度任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
            return;
        }
        //2.1 预警预警 如果正常了 要改会正常
        List<ProjectEarlyWarning> processWarning = trueWarnings.stream().filter(w -> Objects.nonNull(w.getProcessWarning())
                        && w.getProcessWarning())
                .collect(Collectors.toList());
        checkProcessWarning(processWarning);

        //2.2 填报预警 如果正常了 要改会正常
        List<ProjectEarlyWarning> declaredWarning = trueWarnings.stream().filter(w ->
                        Objects.nonNull(w.getDeclaredWarning())
                        && w.getDeclaredWarning())
                .collect(Collectors.toList());
        checkDeclaredWarning(declaredWarning);

        //2.3 实施预警 如果正常了 要改会正常
        List<ProjectEarlyWarning> operationWarning = trueWarnings.stream().filter(w ->
                        Objects.nonNull(w.getOperationWarning())
                        && w.getOperationWarning())
                .collect(Collectors.toList());
        checkOperationWarning(operationWarning);

        //2.4 续建资金预警 如果正常了 要改会正常
        List<ProjectEarlyWarning> renewalFundWarning = trueWarnings.stream().filter(w ->
                        Objects.nonNull(w.getRenewalFundWarning())
                        && w.getRenewalFundWarning())
                .collect(Collectors.toList());
        checkRenewalFundWarning(renewalFundWarning);


        stopWatch.stop();
        log.info("=========== 预警项目维度任务结束 耗时{}s", stopWatch.getTotalTimeSeconds());
    }

    private void checkRenewalFundWarning(List<ProjectEarlyWarning> renewalFundWarning) {

    }

    private void checkOperationWarning(List<ProjectEarlyWarning> operationWarning) {

    }

    private void checkDeclaredWarning(List<ProjectEarlyWarning> declaredWarning) {

    }

    private void checkProcessWarning(List<ProjectEarlyWarning> processWarning) {

    }
}
